@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2020 Jonathan Müller <jonathanmueller.dev@gmail.com>
+// Copyright (C) 2015-2021 Müller <jonathanmueller.dev@gmail.com>
 // This file is subject to the license terms in the LICENSE file
 // found in the top-level directory of this distribution.
 
@@ -65,9 +65,7 @@ namespace foonathan
                                                         std::size_t number_of_nodes) noexcept
             {
                 return detail::memory_block_stack::implementation_offset()
-                       + number_of_nodes
-                             * (((node_size > min_node_size) ? node_size : min_node_size)
-                                + (detail::debug_fence_size ? 2 * detail::max_alignment : 0));
+                       + free_list::min_block_size(node_size, number_of_nodes);
             }
 
             /// \effects Creates it by specifying the size each \concept{concept_node,node} will have,
@@ -76,7 +74,7 @@ namespace foonathan
             /// It will allocate an initial memory block with given size from the \concept{concept_blockallocator,BlockAllocator}
             /// and puts it onto the free list.
             /// \requires \c node_size must be a valid \concept{concept_node,node size}
-            /// and \c block_size must be a non-zero value.
+            /// and \c block_size must be at least \c min_block_size(node_size, 1).
             template <typename... Args>
             memory_pool(std::size_t node_size, std::size_t block_size, Args&&... args)
             : arena_(block_size, detail::forward<Args>(args)...), free_list_(node_size)
