@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2020 Jonathan Müller <jonathanmueller.dev@gmail.com>
+// Copyright (C) 2015-2021 Müller <jonathanmueller.dev@gmail.com>
 // This file is subject to the license terms in the LICENSE file
 // found in the top-level directory of this distribution.
 
@@ -180,12 +180,24 @@ namespace foonathan
                 auto mem = pool.empty() ? nullptr : pool.allocate(count * node_size);
                 if (!mem)
                 {
-                    // use stack for allocation
-                    detail::check_allocation_size<bad_array_size>(
-                        count * node_size, [&] { return next_capacity() - pool.alignment() + 1; },
-                        info());
-                    mem = reserve_memory(pool, count * node_size).memory;
-                    FOONATHAN_MEMORY_ASSERT(mem);
+                    // reserve more memory
+                    auto block = reserve_memory(pool, def_capacity());
+                    pool.insert(block.memory, block.size);
+
+                    mem = pool.allocate(count * node_size);
+                    if (!mem)
+                    {
+                        // reserve more then the default capacity if that didn't work either
+                        detail::check_allocation_size<bad_array_size>(
+                            count * node_size,
+                            [&] { return next_capacity() - pool.alignment() + 1; }, info());
+
+                        block = reserve_memory(pool, count * node_size);
+                        pool.insert(block.memory, block.size);
+
+                        mem = pool.allocate(count * node_size);
+                        FOONATHAN_MEMORY_ASSERT(mem);
+                    }
                 }
 
                 return mem;
