@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2020 Jonathan Müller <jonathanmueller.dev@gmail.com>
+// Copyright (C) 2015-2021 Müller <jonathanmueller.dev@gmail.com>
 // This file is subject to the license terms in the LICENSE file
 // found in the top-level directory of this distribution.
 
@@ -31,8 +31,8 @@ namespace
             // last is inclusive, so add actual_size to it
             // note: cannot use next, might not be directly after
             auto end = last + node_size;
-            FOONATHAN_MEMORY_ASSERT((end - first) % node_size == 0u);
-            return (end - first) / node_size;
+            FOONATHAN_MEMORY_ASSERT(static_cast<std::size_t>(end - first) % node_size == 0u);
+            return static_cast<std::size_t>(end - first) / node_size;
         }
     };
 
@@ -131,17 +131,14 @@ free_memory_list::free_memory_list(std::size_t node_size) noexcept
 {
 }
 
-free_memory_list::free_memory_list(std::size_t node_size, void* mem,
-                                   std::size_t size) noexcept
+free_memory_list::free_memory_list(std::size_t node_size, void* mem, std::size_t size) noexcept
 : free_memory_list(node_size)
 {
     insert(mem, size);
 }
 
 free_memory_list::free_memory_list(free_memory_list&& other) noexcept
-: first_(other.first_),
-  node_size_(other.node_size_),
-  capacity_(other.capacity_)
+: first_(other.first_), node_size_(other.node_size_), capacity_(other.capacity_)
 {
     other.first_    = nullptr;
     other.capacity_ = 0u;
@@ -177,7 +174,7 @@ void* free_memory_list::allocate() noexcept
 
     auto mem = first_;
     first_   = list_get_next(first_);
-    return debug_fill_new(mem, node_size_, fence_size());
+    return detail::debug_fill_new(mem, node_size_, 0);
 }
 
 void* free_memory_list::allocate(std::size_t n) noexcept
@@ -186,9 +183,7 @@ void* free_memory_list::allocate(std::size_t n) noexcept
     if (n <= node_size_)
         return allocate();
 
-    auto actual_size = node_size_ + 2 * fence_size();
-
-    auto i = list_search_array(first_, n + 2 * fence_size(), actual_size);
+    auto i = list_search_array(first_, n, node_size_);
     if (i.first == nullptr)
         return nullptr;
 
@@ -196,16 +191,16 @@ void* free_memory_list::allocate(std::size_t n) noexcept
         list_set_next(i.prev, i.next); // change next from previous to first after
     else
         first_ = i.next;
-    capacity_ -= i.size(actual_size);
+    capacity_ -= i.size(node_size_);
 
-    return debug_fill_new(i.first, n, fence_size());
+    return detail::debug_fill_new(i.first, n, 0);
 }
 
 void free_memory_list::deallocate(void* ptr) noexcept
 {
     ++capacity_;
 
-    auto node = static_cast<char*>(debug_fill_free(ptr, node_size_, fence_size()));
+    auto node = static_cast<char*>(detail::debug_fill_free(ptr, node_size_, 0));
     list_set_next(node, first_);
     first_ = node;
 }
@@ -216,8 +211,8 @@ void free_memory_list::deallocate(void* ptr, std::size_t n) noexcept
         deallocate(ptr);
     else
     {
-        auto mem = debug_fill_free(ptr, n, fence_size());
-        insert_impl(mem, n + 2 * fence_size());
+        auto mem = detail::debug_fill_free(ptr, n, 0);
+        insert_impl(mem, n);
     }
 }
 
@@ -226,23 +221,16 @@ std::size_t free_memory_list::alignment() const noexcept
     return alignment_for(node_size_);
 }
 
-std::size_t free_memory_list::fence_size() const noexcept
-{
-    // fence size is max alignment
-    return debug_fence_size ? max_alignment : 0u;
-}
-
 void free_memory_list::insert_impl(void* mem, std::size_t size) noexcept
 {
-    auto actual_size = node_size_ + 2 * fence_size();
-    auto no_nodes    = size / actual_size;
+    auto no_nodes = size / node_size_;
     FOONATHAN_MEMORY_ASSERT(no_nodes > 0);
 
     auto cur = static_cast<char*>(mem);
     for (std::size_t i = 0u; i != no_nodes - 1; ++i)
     {
-        list_set_next(cur, cur + actual_size);
-        cur += actual_size;
+        list_set_next(cur, cur + node_size_);
+        cur += node_size_;
     }
     list_set_next(cur, first_);
     first_ = static_cast<char*>(mem);
@@ -300,8 +288,9 @@ namespace
             else if (less(cur_backward, memory))
                 // the next position is the previous backwards pointer
                 return {cur_backward, prev_backward};
-            debug_check_double_dealloc(
-                [&] { return cur_forward != memory && cur_backward != memory; }, info, memory);
+            debug_check_double_dealloc([&]
+                                       { return cur_forward != memory && cur_backward != memory; },
+                                       info, memory);
             xor_list_iter_next(cur_forward, prev_forward);
             xor_list_iter_next(cur_backward, prev_backward);
         } while (less(prev_forward, prev_backward));
@@ -353,9 +342,8 @@ ordered_free_memory_list::ordered_free_memory_list(std::size_t node_size) noexce
     xor_list_set(end_node(), begin_node(), nullptr);
 }
 
-ordered_free_memory_list::ordered_free_memory_list(ordered_free_memory_list&& other)
-    noexcept : node_size_(other.node_size_),
-                         capacity_(other.capacity_)
+ordered_free_memory_list::ordered_free_memory_list(ordered_free_memory_list&& other) noexcept
+: node_size_(other.node_size_), capacity_(other.capacity_)
 {
     if (!other.empty())
     {
@@ -432,7 +420,7 @@ void ordered_free_memory_list::insert(void* mem, std::size_t size) noexcept
 {
     FOONATHAN_MEMORY_ASSERT(mem);
     FOONATHAN_MEMORY_ASSERT(is_aligned(mem, alignment()));
-    debug_fill_internal(mem, size, false);
+    detail::debug_fill_internal(mem, size, false);
 
     insert_impl(mem, size);
 }
@@ -456,8 +444,14 @@ void* ordered_free_memory_list::allocate() noexcept
         last_dealloc_ = next;
         FOONATHAN_MEMORY_ASSERT(last_dealloc_prev_ == prev);
     }
+    else if (node == last_dealloc_prev_)
+    {
+        // now the previous node is the node before ours
+        last_dealloc_prev_ = prev;
+        FOONATHAN_MEMORY_ASSERT(last_dealloc_ == next);
+    }
 
-    return debug_fill_new(node, node_size_, fence_size());
+    return detail::debug_fill_new(node, node_size_, 0);
 }
 
 void* ordered_free_memory_list::allocate(std::size_t n) noexcept
@@ -467,30 +461,35 @@ void* ordered_free_memory_list::allocate(std::size_t n) noexcept
     if (n <= node_size_)
         return allocate();
 
-    auto actual_size = node_size_ + 2 * fence_size();
-
-    auto i = xor_list_search_array(begin_node(), end_node(), n + 2 * fence_size(), actual_size);
+    auto i = xor_list_search_array(begin_node(), end_node(), n, node_size_);
     if (i.first == nullptr)
         return nullptr;
 
     xor_list_change(i.prev, i.first, i.next); // change next pointer from i.prev to i.next
     xor_list_change(i.next, i.last, i.prev);  // change prev pointer from i.next to i.prev
-    capacity_ -= i.size(actual_size);
+    capacity_ -= i.size(node_size_);
 
     // if last_dealloc_ points into the array being removed
-    if (less_equal(i.first, last_dealloc_) && less_equal(last_dealloc_, i.last))
+    if ((less_equal(i.first, last_dealloc_) && less_equal(last_dealloc_, i.last)))
     {
         // move last_dealloc just outside range
         last_dealloc_      = i.next;
         last_dealloc_prev_ = i.prev;
     }
+    // if the previous deallocation is the last element of the array
+    else if (last_dealloc_prev_ == i.last)
+    {
+        // it is now the last element before the array
+        FOONATHAN_MEMORY_ASSERT(last_dealloc_ == i.next);
+        last_dealloc_prev_ = i.prev;
+    }
 
-    return debug_fill_new(i.first, n, fence_size());
+    return detail::debug_fill_new(i.first, n, 0);
 }
 
 void ordered_free_memory_list::deallocate(void* ptr) noexcept
 {
-    auto node = static_cast<char*>(debug_fill_free(ptr, node_size_, fence_size()));
+    auto node = static_cast<char*>(debug_fill_free(ptr, node_size_, 0));
 
     auto p =
         find_pos(allocator_info(FOONATHAN_MEMORY_LOG_PREFIX "::detail::ordered_free_memory_list",
@@ -510,8 +509,8 @@ void ordered_free_memory_list::deallocate(void* ptr, std::size_t n) noexcept
         deallocate(ptr);
     else
     {
-        auto mem  = debug_fill_free(ptr, n, fence_size());
-        auto prev = insert_impl(mem, n + 2 * fence_size());
+        auto mem  = detail::debug_fill_free(ptr, n, 0);
+        auto prev = insert_impl(mem, n);
 
         last_dealloc_      = static_cast<char*>(mem);
         last_dealloc_prev_ = prev;
@@ -523,16 +522,9 @@ std::size_t ordered_free_memory_list::alignment() const noexcept
     return alignment_for(node_size_);
 }
 
-std::size_t ordered_free_memory_list::fence_size() const noexcept
-{
-    // node size is fence size
-    return debug_fence_size ? node_size_ : 0u;
-}
-
 char* ordered_free_memory_list::insert_impl(void* mem, std::size_t size) noexcept
 {
-    auto actual_size = node_size_ + 2 * fence_size();
-    auto no_nodes    = size / actual_size;
+    auto no_nodes = size / node_size_;
     FOONATHAN_MEMORY_ASSERT(no_nodes > 0);
 
     auto p =
@@ -541,7 +533,7 @@ char* ordered_free_memory_list::insert_impl(void* mem, std::size_t size) noexcep
                  static_cast<char*>(mem), begin_node(), end_node(), last_dealloc_,
                  last_dealloc_prev_);
 
-    xor_link_block(mem, actual_size, no_nodes, p.prev, p.next);
+    xor_link_block(mem, node_size_, no_nodes, p.prev, p.next);
     capacity_ += no_nodes;
 
     if (p.prev == last_dealloc_prev_)
