@@ -1,10 +1,10 @@
-// Copyright (C) 2015-2020 Jonathan Müller <jonathanmueller.dev@gmail.com>
+// Copyright (C) 2015-2021 Müller <jonathanmueller.dev@gmail.com>
 // This file is subject to the license terms in the LICENSE file
 // found in the top-level directory of this distribution.
 
 #include "aligned_allocator.hpp"
 
-#include <catch.hpp>
+#include <doctest/doctest.h>
 
 #include "detail/align.hpp"
 #include "allocator_storage.hpp"
@@ -12,7 +12,7 @@
 
 using namespace foonathan::memory;
 
-TEST_CASE("aligned_allocator", "[adapter]")
+TEST_CASE("aligned_allocator")
 {
     using allocator_t = aligned_allocator<allocator_reference<memory_stack<>>>;
 
